import asyncio
import logging
import pprint

from tomah import Client


async def amain():
    # set the logging level to DEBUG
    logging.basicConfig(level=logging.DEBUG)
    t = Client(
    )
    await t.regions()
    print(f"me: {pprint.pformat(await t.me())}")


if __name__ == "__main__":
    loop = asyncio.new_event_loop()
    asyncio.set_event_loop(loop)
    try:
        asyncio.run(amain())
    except KeyboardInterrupt:
        pass



import logging
from typing import Optional

import voluptuous as vol
from homeassistant import config_entries

from .const import DOMAIN

_LOGGER = logging.getLogger(__name__)


class TomahConfigFlow(config_entries.ConfigFlow, domain=DOMAIN):
    VERSION = 1
    MINOR_VERSON = 1

    def __init__(self):
        pass

    async def async_step_user(self, user_input: Optional[dict] = None):
        if user_input is not None:
            _LOGGER.warning(f"config info: {user_input}")
            return self.async_create_entry(title=DOMAIN, data=user_input)

        return self.async_show_form(
            step_id="user",
            data_schema=vol.Schema(
                {
                    vol.Required("username"): str,
                    vol.Required("password"): str,
                    vol.Required("client_id"): str,
                    vol.Required("domain"): str,
                }
            ),
        )

    async def async_step_reconfigure(self, user_input):
        if user_input is not None:
            _LOGGER.warning(f"config info: {user_input}")
            return self.async_create_entry(title=DOMAIN, data=user_input)

        return self.async_show_form(
            step_id="reconfigure",
            data_schema=vol.Schema(
                {
                    vol.Required("username"): str,
                    vol.Required("password"): str,
                    vol.Required("client_id"): str,
                    vol.Required("domain"): str,
                }
            ),
        )



#!/usr/bin/env python
import asyncio
import logging

from homeassistant.helpers.entity import Entity

from tomah import Client
from .const import ATTR_NAME, DEFAULT_NAME, STORAGE_VERSION, STORAGE_KEY, DOMAIN, DATA_STORAGE, DATA_ACCOUNTS

# pylint: disable=W0105
"""Tomah integration"""
# pylint: enable=W0105

_LOGGER = logging.getLogger(__name__)


async def async_setup_entry(hass, entry):
    _LOGGER.debug(f"setup_entry {entry.data}")
    if DOMAIN in hass.data:
        _LOGGER.info("DOMAIN data exists")
        return True
    user_id = entry.data["username"]
    hass.data[DOMAIN] = {
        DATA_STORAGE: hass.helpers.storage.Store(STORAGE_VERSION, STORAGE_KEY),
        DATA_ACCOUNTS: {},
    }

    async def async_load_oauth():
        data = await hass.helpers.storage.Store(STORAGE_VERSION, STORAGE_KEY).async_load()
        if data and user_id in data:
            _LOGGER.debug(f"loaded oauth for {user_id}: {data[user_id]}")
            return data[user_id]
        _LOGGER.debug(f"no saved oauth for {user_id}")
        return None
    #
    # def save_oauth(oauth_data):
    #     _LOGGER.debug("saving oauth")
    #     return asyncio.run_coroutine_threadsafe(async_save_oauth(oauth_data), hass.loop).result()

    async def async_save_oauth(oauth_data):
        _LOGGER.debug(f"saving oauth for {user_id}: {oauth_data}")
        data = await hass.helpers.storage.Store(STORAGE_VERSION, STORAGE_KEY).async_load()
        data[user_id] = oauth_data
        await hass.helpers.storage.Store(STORAGE_VERSION, STORAGE_KEY).async_save(data)

    client = Client(
        entry.data["client_id"],
        entry.data["domain"],
        entry.data["username"],
        entry.data["password"],
        async_load_oauth=async_load_oauth,
        async_save_oauth=async_save_oauth,
    )

    await client.async_login()
    _LOGGER.info("login complete")

    regions = await client.regions()

    _LOGGER.debug(f"regions: {regions}")
    # TODO: check account validity
    hass.data[DOMAIN][DATA_ACCOUNTS][entry.data["username"]] = client

    async def refresh_oauth_task():
        _LOGGER.debug(f"start refresh task for {client}")
        while True:
            await asyncio.sleep(60)
            await client.check_auth_refresh()

    hass.async_create_task(refresh_oauth_task())

    platforms = client.get_hass_platforms()

    # TODO: use consts
    await hass.config_entries.async_forward_entry_setups(entry, platforms)
    return True
# class TomahAuthRefresher(DataUpdateCoordinator):
#     """Refresh access token"""
#
#     def __init__(self, hass, check_refresh_cb):
#         super().__init__(hass, _LOGGER, name=self.__class__.__name__, update_interval=timedelta(seconds=30))
#
#         self.async_check_refresh_cb = check_refresh_cb
#
#     async def _async_update_data(self):
#         _LOGGER.debug("checking auth refresh")
#         await self.async_check_refresh_cb()
#


class TomahEntity(Entity):
    pass



import logging

from homeassistant.components.button import ButtonEntity

from . import TomahEntity
from .const import DOMAIN, DATA_ACCOUNTS

_LOGGER = logging.getLogger(__name__)


async def async_setup_entry(hass, config_entry, async_add_entities):
    _LOGGER.info(f"setup entry Button {config_entry.data}")
    client = hass.data[DOMAIN][DATA_ACCOUNTS][config_entry.data["username"]]

    if not client:
        _LOGGER.warning(f"asked to setup Button for {config_entry.data['username']} but no client found")
        return
    entities = []
    for a in client.access_points():
        _LOGGER.info(f"adding door release for {a}")
        entities.append(TomahDoorRelease(client, a))
    async_add_entities(entities)


class TomahDoorRelease(TomahEntity, ButtonEntity):
    _attr_has_entity_name = True
    _attr_icon = "mdi:door"

    def __init__(self, client, access_point):
        # TODO: fetch client from hass.data or receive it here, and/or pass it to superclass?
        self._client = client
        self.access_point = access_point
        self._attr_unique_id = f"{access_point['unit_id']}_{access_point['device_id']}"
        self._attr_name = self.access_point["name"]
        _LOGGER.info("instantiating TomahDoorRelease")
        super().__init__()

    async def async_press(self):
        _LOGGER.info(f"pressing button for {self.access_point}")
        resp = await self._client.door_release(self.access_point)
        _LOGGER.debug(f"door release response: {resp}")



import logging
import time
from urllib.parse import urljoin

import httpx
from ratelimit import limits

_LOGGER = logging.getLogger(__name__)


class Auth(httpx.Auth):
    ACCESS_TOKEN_RENEWAL_PCT = 0.8

    def __init__(
        self,
        client_id,
        url_base,
        username,
        password,
        session=None,
        async_load_oauth=None,
        async_save_oauth=None,
        token_renewal_pct=ACCESS_TOKEN_RENEWAL_PCT,
    ):
        self._client_id = client_id
        self._url_base = url_base
        self._username = username
        self._password = password
        self._async_load_oauth_cb = async_load_oauth
        self._async_save_oauth_cb = async_save_oauth
        self.token_renewal_pct = token_renewal_pct

        if session:
            self._session = session
        else:
            self._session = httpx.AsyncClient()
        self._oauth = None

    # TODO: use aiohttp instead of requests?
    async def async_login(self):
        _LOGGER.debug("async_login")
        if self._async_load_oauth_cb:
            self._oauth = await self._async_load_oauth_cb()
            # TODO: force refresh of token regardless of whether it's expired?
            if self.should_refresh_access_token():
                _LOGGER.debug("async_login: access_token expired, refreshing access token")
                if not await self.refresh_access_token():
                    _LOGGER.debug("async_login: can't refresh access token, re-authenticating")
                    await self._authenticate()
        else:
            await self._authenticate()

    async def _save_oauth(self):
        if self._async_save_oauth_cb:
            await self._async_save_oauth_cb(self._oauth)

    def access_token_expired(self):
        if not self._oauth:
            return True
        if "expires_in" not in self._oauth or "created_at" not in self._oauth:
            return True
        if self._oauth["created_at"] + self._oauth["expires_in"] < time.time():
            _LOGGER.debug("token expired")
            return True
        return False

    # limit to 5 calls per minute
    @limits(calls=5, period=60)
    async def refresh_access_token(self):
        _LOGGER.debug("refresh access token")
        if not self.access_token:
            _LOGGER.info("no valid refresh token, can't refresh access token")
            return False
        resp = await self._session.post(
            urljoin(self._url_base, "oauth/token"),
            data={
                "grant_type": "refresh_token",
                "client_id": self._client_id,
                "refresh_token": self._oauth["refresh_token"],
            },
            headers={"Authorization": f"Bearer {self.access_token}"},
        )

        # tell caller to try and re-login
        if resp.status_code != 200:
            _LOGGER.info(f"got status code {resp.status_code} from refresh token, try to re-login? {resp.text}")
            return False
        _LOGGER.info(f"refreshed access token: {resp.text}")
        self._oauth = resp.json()
        await self._save_oauth()
        return True

    def should_refresh_access_token(self):
        if not self._oauth:
            return True
        if "expires_in" not in self._oauth or "created_at" not in self._oauth:
            return True
        if self._oauth["created_at"] + (self._oauth["expires_in"] * self.token_renewal_pct) < time.time():
            return True
        return False

    async def refresh_access_token_if_needed(self):
        if self.should_refresh_access_token():
            await self.refresh_access_token()

    # limit to 5 calls per minute
    @limits(calls=5, period=60)
    async def _authenticate(self):
        resp = await self._session.post(
            urljoin(self._url_base, "oauth/token"),
            data={
                "client_id": self._client_id,
                "grant_type": "password",
                "username": self._username,
                "password": self._password,
            },
        )
        if resp.status_code != 200:
            raise Exception(f"Failed to login: {resp.text}")
        self._oauth = resp.json()
        _LOGGER.debug(f"authentication successful {self._oauth}")
        await self._save_oauth()
        return True

    def valid_token(self):
        if not self._oauth:
            return False
        if self.access_token_expired():
            return False
        return True

    @property
    def access_token(self):
        if not self._oauth or "access_token" not in self._oauth:
            return None
        return self._oauth["access_token"]

    async def async_auth_flow(self, request):
        if self.should_refresh_access_token():
            await self.refresh_access_token()
            if not self.valid_token():
                await self._authenticate()
            if not self.valid_token():
                raise Exception("Failed to get a valid token")
        request.headers.update({"Authorization": f"Bearer {self.access_token}"})
        # print(self.access_token)
        yield request



import logging
from urllib.parse import urljoin

import httpx
import json_api_doc

from .auth import Auth

_LOGGER = logging.getLogger(__name__)


class Client:
    ACCOUNTS_URL_PREFIX = "https://accounts."
    API_URL_PREFIX = "https://api."
    USER_AGENT_SUFFIX = "/986 CFNetwork/897.15 Darwin/17.5.0"

    def __init__(
        self,
        client_id,
        domain,
        username,
        password,
        async_load_oauth=None,
        async_save_oauth=None,
        token_renewal_pct=None,
    ):
        self.units = None

        domain_components = domain.split(".")

        self._auth_session = httpx.AsyncClient(headers={"User-Agent": domain_components[0] + self.USER_AGENT_SUFFIX})

        self._api_url_base = self.API_URL_PREFIX + domain
        self._accounts_url_base = self.ACCOUNTS_URL_PREFIX + domain
        kwargs = {}
        if token_renewal_pct is not None:
            kwargs["token_renewal_pct"] = token_renewal_pct
        self._auth = Auth(
            client_id,
            self._accounts_url_base,
            username,
            password,
            session=self._auth_session,
            async_load_oauth=async_load_oauth,
            async_save_oauth=async_save_oauth,
            **kwargs,
        )
        self._session = httpx.AsyncClient(
            headers={"User-Agent": domain_components[0] + self.USER_AGENT_SUFFIX}, auth=self._auth
        )

    async def async_login(self):
        await self._auth.async_login()
        await self.query_units()

    def get_hass_platforms(self):
        return ["button"]

    async def check_auth_refresh(self):
        """Provide callback to refresh access token if needed."""
        await self._auth.refresh_access_token_if_needed()

    # def login(self):
    #     self._oauth = self.get_oauth()
    #     print(self._oauth)

    async def query_units(self):
        me = await self.me()
        self.units = me["units"]

    def access_points(self):
        for unit in self.units:
            for access_point in unit["access_points"]:
                yield {
                    "name": access_point["name"],
                    "unit_id": unit["id"],
                    "device_id": access_point["device_id"],
                    "device_type": access_point["device_type"],
                }

    async def regions(self):
        resp = await self._session.get(urljoin(self._accounts_url_base, "api/mobile/regions"))
        _LOGGER.debug(f"regions: {resp.text}")
        return resp.json()

    async def tokens(self):
        """Deprecated."""
        resp = await self._session.get(urljoin(self._accounts_url_base, "api/v1/account/tokens"))
        print(f"tokens: {resp.text}")

    async def devices(self):
        resp = await self._session.get(urljoin(self._api_url_base, "mobile/v4/users/devices"))
        print(f"devices: {resp.text}")

    async def me(self):
        resp = await self._session.get(urljoin(self._api_url_base, "mobile/v3/me"))
        return json_api_doc.deserialize(resp.json())
        # return resp.json()

    # data[type]=door_release_requests
    # data[relationships][unit][data][id]=931502
    # data[relationships][device][data][type]=panels
    # data[relationships][device][data][id]=12064
    # data[attributes][release_method]=front_door_view

    async def door_release(self, access_point):
        if "unit_id" not in access_point or "device_id" not in access_point or "device_type" not in access_point:
            # TODO: raise exception?
            return None

        door_release_requests_payload = {
            "data[type]": "door_release_requests",
            "data[relationships][unit][data][id]": access_point["unit_id"],
            "data[relationships][device][data][type]": access_point["device_type"],
            "data[relationships][device][data][id]": access_point["device_id"],
            "data[attributes][release_method]": "front_door_view",
        }

        resp = await self._session.post(
            urljoin(self._api_url_base, "mobile/v3/door_release_requests"),
            data=door_release_requests_payload,
        )

        if resp.status_code < 200 or resp.status_code >= 300:
            _LOGGER.error(f"Failed to create door release request: {resp.text}")
            return None

        return json_api_doc.deserialize(resp.json())

    # def get_oauth(self):
    #     resp = self._session.post(
    #         urljoin(self.ACCOUNTS_URL_PREFIX, "oauth/token"),
    #         data={
    #             "client_id": self.CLIENT_ID,
    #             "grant_type": "password",
    #             "username": self.USERNAME,
    #             "password": self.PASSWORD,
    #         },
    #     )
    #     # print(resp)
    #     return resp.json()
